/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;



import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;

import vocleditor.action.request.VOCLContainerPropertiesRequest;
import vocleditor.action.request.VOCLExpNegateRequest;
import vocleditor.commands.adds.AddConditionCommand;
import vocleditor.commands.property.VOCLExpNegateCommand;
import vocleditor.edit.policy.NavExpContainerXYLayoutEditPolicy;
import vocleditor.edit.policy.VOCLEditPolicy;
import vocleditor.figure.NavExpContainerFigure;
import VOCLModel.Condition;
import VOCLModel.NavExpContainer;
import VOCLModel.VOCLExpression;
import VOCLModel.VOCLModel4Package;



/**
 * Class       :NavExpContainerEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for NavExpContainer
 * <br/>
 * @author vilapower
 */
public class NavExpContainerEditPart extends AbstractExpressionEditPart  {

	/**
	 * Creates a new NavExpContainerEditPart instance.
	 * @param element NavExpContainer
	 */
	public NavExpContainerEditPart(NavExpContainer container) {
		setModel(container);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure(){
		boolean negated = this.getNavExpContainer().isIsNegated();
		NavExpContainerFigure figure=new NavExpContainerFigure(negated);
		return figure;
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies(){
		installEditPolicy(EditPolicy.CONTAINER_ROLE, new VOCLEditPolicy());
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new NavExpContainerXYLayoutEditPolicy());
		super.createEditPolicies();	
	}

	/**
	 * returns model of this EditPart
	 * @return NavExpContainer 
	 */
	public NavExpContainer getNavExpContainer(){
		return (NavExpContainer) getModel();
	}
    
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren(){
		List	allChildren = new LinkedList();
		allChildren.addAll(getNavExpContainer().getClassifierNode());
		allChildren.addAll(getNavExpContainer().getCondition());
		if(getNavExpContainer().getPredicateContainer()!=null)
			allChildren.add(getNavExpContainer().getPredicateContainer());
		return allChildren;
	}

	/**
	 * return Figure of this EditPart
	 * @return NavExpContainerFigure
	 */
	public NavExpContainerFigure getNavExpContainerFigure(){
		return (NavExpContainerFigure)getFigure();
	}

	/**
	 * @see vocleditor.edit.AbstractExpressionEditPart#getPrivateContentPane(org.eclipse.gef.EditPart)
	 */
	public IFigure getPrivateContentPane(EditPart child) {
		if(child instanceof ConditionEditPart)
			return getNavExpContainerFigure().getConditionPane();
		else 
			return getNavExpContainerFigure().getFreeFormPane();	
	}
	
	/**
	 * @see vocleditor.edit.AbstractExpressionEditPart#getPrivateContentPane(java.lang.Object)
	 */
	public IFigure getPrivateContentPane(Object child) {
		if(child instanceof Condition)
			return getNavExpContainerFigure().getConditionPane();
		else 
			return getNavExpContainerFigure().getFreeFormPane();	
	}
	
	/**
	 * @see org.eclipse.gef.EditPart#understandsRequest(org.eclipse.gef.Request)
	 */
	public boolean understandsRequest(Request req) {
		if (req instanceof VOCLExpNegateRequest) return true;
		if (req instanceof VOCLContainerPropertiesRequest) return true;
		return super.understandsRequest(req);
	}

	/**
	 * @see org.eclipse.gef.EditPart#getCommand(org.eclipse.gef.Request)
	 */
	public Command getCommand(Request request) {
		if (request instanceof VOCLContainerPropertiesRequest) return new AddConditionCommand();
		if (request instanceof VOCLExpNegateRequest) return new VOCLExpNegateCommand();
		return super.getCommand(request);
	}



	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshChildren()
	 */
	protected void refreshChildren() {		
		int i;
		EditPart editPart;

		Map modelOfConditionToEditPart = new HashMap();
		Map modelOfOtherToEditPart = new HashMap();
		
		List children = getChildren();
		
		List childrenAsCondition=new LinkedList();
		List childrenAsOther=new LinkedList();
		
		for (i = 0; i < children.size(); i++) {
			editPart = (EditPart)children.get(i);
			if(editPart instanceof ConditionEditPart){
				modelOfConditionToEditPart.put(editPart.getModel(), editPart);
				childrenAsCondition.add(editPart);
			}else{
				modelOfOtherToEditPart.put(editPart.getModel(), editPart);
				childrenAsOther.add(editPart);
			}
		}
		
		List modelObjectsOfCondition =new LinkedList();
		List modelObjectsOfOther =new LinkedList();
		Iterator it=getModelChildren().iterator();
		while(it.hasNext()){
			Object model=it.next();
			if(model instanceof Condition)
				modelObjectsOfCondition.add(model);
			else
				modelObjectsOfOther.add(model);
		}
		refreshPrivateChildren(modelObjectsOfCondition,modelOfConditionToEditPart,childrenAsCondition);
		refreshPrivateChildren(modelObjectsOfOther,modelOfOtherToEditPart,childrenAsOther);
	}
	
	/**
	 * @see vocleditor.edit.AbstractExpressionEditPart#getExpression()
	 */
	public VOCLExpression getExpression() {
		return getNavExpContainer();
	}

	/**
	 * checks if the model was be changed
	* @param notification org.eclipse.emf.common.notify.Notification
	*/	
	public void notifyChanged(Notification notification) {
		int type = notification.getEventType();
		int featureId = notification.getFeatureID(VOCLModel4Package.class);
		
		if(featureId == VOCLModel4Package.NAV_EXP_CONTAINER__IS_NEGATED){
			refreshVisuals();
		}
		else 
			super.notifyChanged(notification);
		
	}
	

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		getNavExpContainerFigure().setNegated(getNavExpContainer().isIsNegated());
		super.refreshVisuals();
		
		
		
	}

	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#setSelected(int)
	 */
	public void setSelected(int value) {}


}
