/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */


package vocleditor.model;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import VOCLModel.VOCLModel4Factory;
import VOCLModel.VOCLModel4Package;
import VOCLModel.VOCLDiagramm;
import VOCLModel.impl.VOCLModel4PackageImpl;

/**
 * Drives the model. Acts as the model entry point.
 * @author Vanderheyden
 *
 */
public class VOCLModelManager {
	/**
	 * For the purpose of the simple editor, a file can only contain a workflow.
	 * In EMF, a resource provides the way to have access to the model content.
	 */
	private Resource resource = null;

	/**
	 * Contains the factory associated with the model.
	 */
	private static VOCLModel4Factory oclFactory = null;

	/**
	 * Gives access to the top level workflow contained in the resource.
	 */
	private VOCLDiagramm cont = null;

	/**
	 * Returns the resource containing the workflow. Uses lazy initialization.
	 * @param path
	 * @return
	 */
	public Resource getResource(IPath path) {
		if (resource == null) {
			ResourceSet resSet = getResourceSet();
			resource =
				resSet.getResource(
					URI.createPlatformResourceURI(path.toString()),
					true);
		}
		return resource;
	}

	/**
	 * Returns the resource containing the workflow. Throw a runtime exception 
	 * if the resource does not exist.
	 * @param path
	 * @return
	 */
	public Resource getResource() {
		if (resource == null) {
			throw new RuntimeException("Ressource supposed to be already created");
		}
		return resource;
	}

	/**
	 * Creates a resource to contain the workflow. The resource file does not exist yet.
	 * @param path
	 * @return
	 */
	private Resource createResource(IPath path) {
		if (resource == null) {
			ResourceSet resSet = getResourceSet();
			resource =
				resSet.createResource(URI.createPlatformResourceURI(path.toString()));
		}
		return resource;
	}

	/**
	 * Returns the resource set.
	 * @param 
	 * @return
	 */
	private ResourceSet getResourceSet() {
		// Initialize the NavExpression package
		VOCLModel4PackageImpl.init();
		// Register the XMI resource factory for the .ocl extension
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map m = reg.getExtensionToFactoryMap();
		m.put("ocl", new XMIResourceFactoryImpl());
		// Obtain a new resource set
		return new ResourceSetImpl();
	}

	/**
	 * Returns the factory associated with the model.
	 * Object creation are made through that factory.
	 * @return
	 */
	static public VOCLModel4Factory getFactory() {
		if (oclFactory == null) {
			// Access the factory (needed to create instances)
			Map registry = EPackage.Registry.INSTANCE;
			String navURI = VOCLModel4Package.eNS_URI;
			VOCLModel4Package navPackage =
				(VOCLModel4Package) registry.get(navURI);
			oclFactory = navPackage.getVOCLModel4Factory();
		}
		return oclFactory;
	}

	/**
	 * Creates a new navexpression.
	 * @param 
	 * @return
	 */
	public VOCLDiagramm createVOCLDiagramm(IPath path) {
		createResource(path);
		// Create a new NavExp model
		Map registry = EPackage.Registry.INSTANCE;
		String navURI = VOCLModel4Package.eNS_URI;
		VOCLModel4Package navExpPackage = (VOCLModel4Package) registry.get(navURI);
		VOCLModel4Factory navExpFactory = navExpPackage.getVOCLModel4Factory();
		cont = navExpFactory.createVOCLDiagramm();
		resource.getContents().add(cont);
		return cont;
	}

	/**
	 * Loads the content of the model from the file.
	 * @param path
	 */
	public void load(IPath path) throws IOException {
		getResource(path);
		Map options = new HashMap();
		options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
		resource.load(options);
	}

    /**
     * reloads the content of the model from the file.
     * @param path
     */
    public void reload(IPath path) throws IOException {
        getResource(path).unload();
        load(path);
    }

	/**
	 * Saves the content of the model to the file.
	 * @param path
	 */
	public void save(IPath path) throws IOException {
		// Nothing is done of the path if the resource already exist...
		// Not really what we would expect, but ok for the NavExpression application.
		// A second save() method with no argument should be added.
		getResource(path);
		Map options = new HashMap();
		options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
		resource.save(options);
	}

	/**
	 * Gets the top level navexpression model.
	 * @return
	 */
	public VOCLDiagramm getModel() {
		if (null == cont) {
			EList l = resource.getContents();
			Iterator i = l.iterator();
			while (i.hasNext()) {
				Object o = i.next();
				if (o instanceof VOCLDiagramm)
				cont = (VOCLDiagramm) o;
			}
		}
		return cont;
	}
}
